@@ -3,11 +3,16 @@ import asyncio
 import aiohttp
 import webbrowser
 import json
+from aiologger.logger import Logger
+from aiologger.levels import LogLevel
 from collections import OrderedDict
 
 from .exceptions import ResponseProcessException
 
 
+logger = Logger.with_default_handlers(name="aiotapioca", level=LogLevel.DEBUG)
+
+
 class TapiocaInstantiator:
     def __init__(self, adapter_class):
         self.adapter_class = adapter_class
@@ -130,7 +135,7 @@ class TapiocaClient:
             return self._wrap_in_tapioca(data=self._data[name])
 
         # if could not access, falback to resource mapping
-        resource_mapping = self._api.resource_mapping
+        resource_mapping = self._api.get_resource_mapping(self._api_params)
         if name in resource_mapping:
             resource = resource_mapping[name]
             api_root = self._api.get_api_root(self._api_params, resource_name=name)
@@ -176,8 +181,9 @@ class TapiocaClient:
         return ret
 
     def __dir__(self):
+        resource_mapping = self._api.get_resource_mapping(self._api_params)
         if self._api and self._data is None:
-            return [key for key in self._api.resource_mapping.keys()]
+            return [key for key in resource_mapping.keys()]
 
         if isinstance(self._data, dict):
             return self._data.keys()
@@ -186,14 +192,14 @@ class TapiocaClient:
 
     def __str__(self):
         if type(self._data) == OrderedDict:
-            return ("<{} object, printing as dict:\n" "{}>").format(
+            return "<{} object, printing as dict:\n" "{}>".format(
                 self.__class__.__name__, json.dumps(self._data, indent=4)
             )
         else:
             import pprint
 
             pp = pprint.PrettyPrinter(indent=4)
-            return ("<{} object\n" "{}>").format(
+            return "<{} object\n" "{}>".format(
                 self.__class__.__name__, pp.pformat(self._data)
             )
 
@@ -251,6 +257,14 @@ class TapiocaClientExecutor(TapiocaClient):
     def refresh_data(self):
         return self._refresh_data
 
+    def __get_semaphore_value(self, kwargs):
+        semaphore = (
+            kwargs.pop("semaphore", None)
+            or self._api_params.get("semaphore")
+            or self._api.semaphore
+        )
+        return semaphore
+
     @staticmethod
     async def _coro_wrap(func, *args, **kwargs):
         if asyncio.iscoroutinefunction(func):
@@ -260,7 +274,7 @@ class TapiocaClientExecutor(TapiocaClient):
         return result
 
     async def _make_request(
-        self, request_method, refresh_token=None, repeat_number=0, *args, **kwargs
+        self, request_method, refresh_token=False, repeat_number=0, *args, **kwargs
     ):
         if "url" not in kwargs:
             kwargs["url"] = self._data
@@ -303,18 +317,13 @@ class TapiocaClientExecutor(TapiocaClient):
                 message=error_message, client=client
             )
 
-            should_refresh_token = (
-                self._api.refresh_token is True
-                or self._api_params.get("refresh_token") is True
-                or refresh_token is True
-            )
             auth_expired = await self._coro_wrap(
                 self._api.is_authentication_expired, tapioca_exception, **context
             )
 
             propagate_exception = True
 
-            if should_refresh_token and auth_expired:
+            if refresh_token and auth_expired:
                 self._refresh_data = await self._coro_wrap(
                     self._api.refresh_authentication, **context
                 )
@@ -350,14 +359,19 @@ class TapiocaClientExecutor(TapiocaClient):
 
     async def _send(self, request_method, *args, **kwargs):
 
-        if "semaphore" in kwargs and "semaphore_class" not in kwargs:
-            kwargs["semaphore_class"] = asyncio.Semaphore(kwargs.pop("semaphore", 10))
+        if "semaphore_class" not in kwargs:
+            semaphore = self.__get_semaphore_value(kwargs)
+            kwargs["semaphore_class"] = asyncio.Semaphore(semaphore)
 
-        debug = kwargs.pop("debug") if "debug" in kwargs else False
-        semaphore = (
-            kwargs.pop("semaphore_class")
-            if "semaphore_class" in kwargs
-            else asyncio.Semaphore()
+        debug = self._api_params.get("debug", False)
+
+        semaphore = kwargs.pop("semaphore_class", asyncio.Semaphore())
+
+        kwargs["refresh_token"] = (
+            kwargs.get("refresh_token") is True
+            or self._api_params.get("refresh_token") is True
+            or self._api.refresh_token is True
+            or False
         )
 
         async with semaphore:
@@ -365,18 +379,22 @@ class TapiocaClientExecutor(TapiocaClient):
 
         if debug:
             executor = response()
-            response_info = "status: {} | url: {} - response data: {}".format(
-                executor.status, executor.response.url, str(executor.data)[:33]
+            text = str(executor.data)
+            if len(text) > 100:
+                text = "{}...".format(str(executor.data)[:333])
+            msg = "status: {} | url: {} - response data: {}".format(
+                executor.status, executor.response.url, text
             )
-            print(response_info)
+            logger.debug(msg)
 
         return response
 
     async def _send_batch(self, request_method, *args, **kwargs):
 
-        data = kwargs.pop("data") if "data" in kwargs else []
+        data = kwargs.pop("data", [])
 
-        kwargs["semaphore_class"] = asyncio.Semaphore(kwargs.pop("semaphore", 10))
+        semaphore = self.__get_semaphore_value(kwargs)
+        kwargs["semaphore_class"] = asyncio.Semaphore(semaphore)
 
         results = await asyncio.gather(
             *[
@@ -438,10 +456,6 @@ class TapiocaClientExecutor(TapiocaClient):
         item_count = 0
 
         while iterator_list:
-            if executor._reached_max_limits(
-                page_count, item_count, max_pages, max_items
-            ):
-                break
             for item in iterator_list:
                 if executor._reached_max_limits(
                     page_count, item_count, max_pages, max_items
@@ -452,6 +466,11 @@ class TapiocaClientExecutor(TapiocaClient):
 
             page_count += 1
 
+            if executor._reached_max_limits(
+                page_count, item_count, max_pages, max_items
+            ):
+                break
+
             next_request_kwargs = await executor._get_iterator_next_request_kwargs()
 
             if not next_request_kwargs:
